@@ -38,7 +38,7 @@ import Data.List (intercalate)
 import Data.Char (toLower, toUpper, isDigit)
 import Paths_gitit (getDataFileName)
 import System.FilePath ((</>))
-import Text.Pandoc hiding (MathML, WebTeX)
+import Text.Pandoc hiding (MathML, WebTeX, MathJax)
 
 forceEither :: Show e => Either e a -> a
 forceEither = either (error . show) id
@@ -99,11 +99,13 @@ extractConfig cp = do
       cfResetPasswordMessage <- get cp "DEFAULT" "reset-password-message"
       cfUseFeed <- get cp "DEFAULT" "use-feed"
       cfBaseUrl <- get cp "DEFAULT" "base-url"
+      cfAbsoluteUrls <- get cp "DEFAULT" "absolute-urls"
       cfWikiTitle <- get cp "DEFAULT" "wiki-title"
       cfFeedDays <- get cp "DEFAULT" "feed-days"
       cfFeedRefreshTime <- get cp "DEFAULT" "feed-refresh-time"
       cfPDFExport <- get cp "DEFAULT" "pdf-export"
       cfPandocUserData <- get cp "DEFAULT" "pandoc-user-data"
+      cfXssSanitize <- get cp "DEFAULT" "xss-sanitize"
       let (pt, lhs) = parsePageType cfDefaultPageType
       let markupHelpFile = show pt ++ if lhs then "+LHS" else ""
       markupHelpPath <- liftIO $ getDataFileName $ "data" </> "markupHelp" </> markupHelpFile
@@ -128,6 +130,7 @@ extractConfig cp = do
         , mathMethod           = case map toLower cfMathMethod of
                                       "jsmath"   -> JsMathScript
                                       "mathml"   -> MathML
+                                      "mathjax"  -> MathJax "https://d3eoax9i5htok0.cloudfront.net/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML"
                                       "google"   -> WebTeX "http://chart.apis.google.com/chart?cht=tx&chl="
                                       _          -> RawTeX
         , defaultLHS           = lhs
@@ -186,13 +189,16 @@ extractConfig cp = do
         , markupHelp           = markupHelpText
         , useFeed              = cfUseFeed
         , baseUrl              = stripTrailingSlash cfBaseUrl
+        , useAbsoluteUrls      = cfAbsoluteUrls
         , wikiTitle            = cfWikiTitle
         , feedDays             = readNumber "feed-days" cfFeedDays
         , feedRefreshTime      = readNumber "feed-refresh-time" cfFeedRefreshTime
         , pdfExport            = cfPDFExport
         , pandocUserData       = if null cfPandocUserData
                                     then Nothing
-                                    else Just cfPandocUserData }
+                                    else Just cfPandocUserData
+        , xssSanitize          = cfXssSanitize
+        }
   case config' of
         Left (ParseError e, e') -> error $ "Parse error: " ++ e ++ "\n" ++ e'
         Left e                  -> error (show e)
